
--[[-----------------------------------------------------------------------------
Ampere
-------------------------------------------------------------------------------]]

local Refresh = function() end
local EDGEGAP, ROWHEIGHT, ROWGAP, GAP = 16, 20, 2, 4
local NUMADDONS = GetNumAddOns()
local GOLD_TEXT = {1.0, 0.82, 0}
local RED_TEXT = {1, 0, 0}
local STATUS_COLORS = setmetatable({
	DISABLED = {157/256, 157/256, 157/256},
	DEP_DISABLED = {157/256, 157/256, 157/256},
	NOT_DEMAND_LOADED = {1, 0.5, 0},
	DEP_NOT_DEMAND_LOADED = {1, 0.5, 0},
	LOAD_ON_DEMAND = {30/256, 1, 0},
	DISABLED_AT_RELOAD = {163/256, 53/256, 238/256},
	DEP_MISSING = {1, 0.5, 0},
}, {__index = function() return RED_TEXT end})
local L = {
	DISABLED_AT_RELOAD = "Disabled on ReloadUI",
	LOAD_ON_DEMAND = "LoD",
}


local enabledstates = setmetatable({}, {
	__index = function(t, i)
		local name, _, _, enabled = GetAddOnInfo(i)
		if name ~= i then return t[name] end

		t[i] = not not enabled -- Looks silly, but ensures we store a boolean
		return enabled
	end
})


-- We have to hook these, GetAddOnInfo doesn't report back the new enabled state
local orig1, orig2, orig3, orig4 = EnableAddOn, DisableAddOn, EnableAllAddOns, DisableAllAddOns
local function posthook(...) Refresh(); return ... end
EnableAddOn = function(addon, ...)
	enabledstates[GetAddOnInfo(addon)] = true
	return posthook(orig1(addon, ...))
end
DisableAddOn = function(addon, ...)
	enabledstates[GetAddOnInfo(addon)] = false
	return posthook(orig2(addon, ...))
end
EnableAllAddOns = function(...)
	for i=1,NUMADDONS do enabledstates[GetAddOnInfo(i)] = true end
	return posthook(orig3(...))
end
DisableAllAddOns = function(...)
	for i=1,NUMADDONS do enabledstates[GetAddOnInfo(i)] = false end
	return posthook(orig4(...))
end


local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
frame.name = "Ampere"
frame:Hide()
frame:SetScript("OnShow", function(frame)
	local function MakeButton(parent)
		local butt = CreateFrame("Button", nil, parent or frame)
		butt:SetWidth(80) butt:SetHeight(22)

		butt:SetHighlightFontObject(GameFontHighlightSmall)
		butt:SetNormalFontObject(GameFontNormalSmall)

		butt:SetNormalTexture("Interface\\Buttons\\UI-Panel-Button-Up")
		butt:SetPushedTexture("Interface\\Buttons\\UI-Panel-Button-Down")
		butt:SetHighlightTexture("Interface\\Buttons\\UI-Panel-Button-Highlight")
		butt:SetDisabledTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
		butt:GetNormalTexture():SetTexCoord(0, 0.625, 0, 0.6875)
		butt:GetPushedTexture():SetTexCoord(0, 0.625, 0, 0.6875)
		butt:GetHighlightTexture():SetTexCoord(0, 0.625, 0, 0.6875)
		butt:GetDisabledTexture():SetTexCoord(0, 0.625, 0, 0.6875)
		butt:GetHighlightTexture():SetBlendMode("ADD")

		return butt
	end


	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	title:SetText("Addon Management Panel")


	local subtitle = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
--~ 	subtitle:SetHeight(32)
	subtitle:SetHeight(35)
	subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subtitle:SetPoint("RIGHT", frame, -32, 0)
	subtitle:SetNonSpaceWrap(true)
	subtitle:SetJustifyH("LEFT")
	subtitle:SetJustifyV("TOP")
--~ 	subtitle:SetMaxLines(3)
	subtitle:SetText("This panel can be used to toggle addons, load Load-on-Demand addons, or reload the UI.  You must reload UI to unload an addon.  Settings are saved on a per-char basis.")

	local rows, anchor = {}
	local function helper(...)
		for i=1,select("#", ...) do
			local dep = select(i, ...)
			local loaded = IsAddOnLoaded(dep) and 1 or 0
			GameTooltip:AddDoubleLine(i == 1 and "Dependencies:" or " ", dep, 1, 0.4, 0, 1, loaded, loaded)
		end
	end
	local function OnEnter(self)
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(self.addon)
		local author = GetAddOnMetadata(self.addon, "Author")
		local version = GetAddOnMetadata(self.addon, "Version")
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:AddLine(title, nil, nil, nil, true)
		GameTooltip:AddLine(notes, 1, 1, 1, true)
		if author then GameTooltip:AddDoubleLine("Author:", author, 1,0.4,0, 1,1,1) end
		if version then GameTooltip:AddDoubleLine("Version:", version, 1,0.4,0, 1,1,1) end
		helper(GetAddOnDependencies(self.addon))
		GameTooltip:Show()
	end
	local function OnLeave() GameTooltip:Hide() end
	local function OnClick(self)
		local addon = self:GetParent().addon
		local enabled = enabledstates[addon]
		PlaySound(enabled and "igMainMenuOptionCheckBoxOff" or "igMainMenuOptionCheckBoxOn")
		if enabled then DisableAddOn(addon) else EnableAddOn(addon) end
		Refresh()
	end
	local function LoadOnClick(self)
		local addon = self:GetParent().addon
		if not select(4,GetAddOnInfo(addon)) then
			EnableAddOn(addon)
			LoadAddOn(addon)
			DisableAddOn(addon)
		else LoadAddOn(addon) end
	end
	for i=1,math.floor((425-22)/(ROWHEIGHT + ROWGAP)) do
		local row = CreateFrame("Button", nil, frame)
		if not anchor then row:SetPoint("TOP", subtitle, "BOTTOM", 0, -16)
		else row:SetPoint("TOP", anchor, "BOTTOM", 0, -ROWGAP) end
		row:SetPoint("LEFT", EDGEGAP, 0)
		row:SetPoint("RIGHT", -EDGEGAP*2-8, 0)
		row:SetHeight(ROWHEIGHT)
		anchor = row
		rows[i] = row


		local check = CreateFrame("CheckButton", nil, row)
		check:SetWidth(ROWHEIGHT+4)
		check:SetHeight(ROWHEIGHT+4)
		check:SetPoint("LEFT")
		check:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
		check:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
		check:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
		check:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
		check:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")
		check:SetScript("OnClick", OnClick)
		row.check = check


		local title = row:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
		title:SetPoint("LEFT", check, "RIGHT", 4, 0)
		row.title = title


		local loadbutton = MakeButton(row)
		loadbutton:SetPoint("RIGHT")
		loadbutton:SetText("Load")
		loadbutton:SetScript("OnClick", LoadOnClick)
		row.loadbutton = loadbutton


		local reason = row:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
		reason:SetPoint("RIGHT", loadbutton, "LEFT", -4, 0)
		reason:SetPoint("LEFT", title, "RIGHT")
		reason:SetJustifyH("RIGHT")
		row.reason = reason

		row:SetScript("OnEnter", OnEnter)
		row:SetScript("OnLeave", OnLeave)
	end


	local offset = 0
	Refresh = function()
		if not frame:IsVisible() then return end
		for i,row in ipairs(rows) do
			if (i + offset) <= NUMADDONS then
				local name, title, notes, enabled, loadable, reason = GetAddOnInfo(i + offset)
				local version = GetAddOnMetadata(i + offset, "Version")
				if version then title = title.. " |cffff6600("..version:trim()..")" end
				local loaded = IsAddOnLoaded(i + offset)
				local lod = IsAddOnLoadOnDemand(i + offset)
				if lod and not loaded and (not reason or reason == "DISABLED") then
					reason = "LOAD_ON_DEMAND"
					row.loadbutton:Show()
					row.loadbutton:SetWidth(45)
				else
					row.loadbutton:Hide()
					row.loadbutton:SetWidth(1)
				end
				if loaded and not enabledstates[name] then reason = "DISABLED_AT_RELOAD" end

				row.check:SetChecked(enabledstates[name])
				row.title:SetText(title)
				row.reason:SetText(reason and (TEXT(_G["ADDON_" .. reason] or L[reason])))
				row.title:SetTextColor(unpack(reason and STATUS_COLORS[reason] or GOLD_TEXT))
				if reason then row.reason:SetTextColor(unpack(STATUS_COLORS[reason])) end
				row.addon = name
				row.notes = notes
				row:Show()
			else
				row:Hide()
			end
		end
	end
	frame:SetScript("OnEvent", Refresh)
	frame:RegisterEvent("ADDON_LOADED")
	frame:SetScript("OnShow", Refresh)
	Refresh()


	local scrollbar = LibStub("tekKonfig-Scroll").new(frame, nil, #rows/2)
	scrollbar:ClearAllPoints()
	scrollbar:SetPoint("TOP", rows[1], 0, -16)
	scrollbar:SetPoint("BOTTOM", rows[#rows], 0, 16)
	scrollbar:SetPoint("RIGHT", -16, 0)
	scrollbar:SetMinMaxValues(0, math.max(0, NUMADDONS-#rows))
	scrollbar:SetValue(0)

	local f = scrollbar:GetScript("OnValueChanged")
	scrollbar:SetScript("OnValueChanged", function(self, value, ...)
		offset = value
		Refresh()
		return f(self, value, ...)
	end)

	frame:EnableMouseWheel()
	frame:SetScript("OnMouseWheel", function(self, val) scrollbar:SetValue(scrollbar:GetValue() - val*#rows/2) end)


	local enableall = MakeButton()
	enableall:SetPoint("BOTTOMLEFT", 16, 16)
	enableall:SetText("Enable All")
	enableall:SetScript("OnClick", EnableAllAddOns)


	local disableall = MakeButton()
	disableall:SetPoint("LEFT", enableall, "RIGHT", 4, 0)
	disableall:SetText("Disable All")
	disableall:SetScript("OnClick", DisableAllAddOns)


	local reload = MakeButton()
	reload:SetPoint("BOTTOMRIGHT", -16, 16)
	reload:SetText("Reload UI")
	reload:SetScript("OnClick", ReloadUI)
end)

InterfaceOptions_AddCategory(frame)

-------------------------------------------------------------------------------
-- Volume
-------------------------------------------------------------------------------

local def_col, def_bg_col = _G.TOOLTIP_DEFAULT_COLOR, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR
local Volumizer = CreateFrame("Frame", "VolumizerPanel", UIParent)
Volumizer:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event] (self, event, ...) end end)
Volumizer:RegisterEvent("PLAYER_ENTERING_WORLD")

local DropDown = CreateFrame("Frame", "Volume_DropDown", UIParent, "UIDropDownMenuTemplate")
DropDown.info = {}

local VOLUMES = {
	["ambience"] = {
		SoundOption	= SoundPanelOptions.Sound_AmbienceVolume,
		VolumeCVar	= "Sound_AmbienceVolume",
		Volume		= AudioOptionsSoundPanelAmbienceVolume,
		EnableCVar	= "Sound_EnableAmbience",
		Enable		= AudioOptionsSoundPanelAmbientSounds,
		Tooltip		= OPTION_TOOLTIP_ENABLE_AMBIENCE,
	},
	["music"] = {
		SoundOption	= SoundPanelOptions.Sound_MusicVolume,
		VolumeCVar	= "Sound_MusicVolume",
		Volume		= AudioOptionsSoundPanelMusicVolume,
		EnableCVar	= "Sound_EnableMusic",
		Enable		= AudioOptionsSoundPanelMusic,
		Tooltip		= OPTION_TOOLTIP_ENABLE_MUSIC,
	},
	["master"] = {
		SoundOption	= SoundPanelOptions.Sound_MasterVolume,
		VolumeCVar	= "Sound_MasterVolume",
		Volume		= AudioOptionsSoundPanelMasterVolume,
		EnableCVar	= "Sound_EnableAllSound",
		Enable		= AudioOptionsSoundPanelEnableSound,
		Tooltip		= OPTION_TOOLTIP_ENABLE_SOUND,
	},
	["sfx"]	= {
		SoundOption	= SoundPanelOptions.Sound_SFXVolume,
		VolumeCVar	= "Sound_SFXVolume",
		Volume		= AudioOptionsSoundPanelSoundVolume,
		EnableCVar	= "Sound_EnableSFX",
		Enable		= AudioOptionsSoundPanelSoundEffects,
		Tooltip		= OPTION_TOOLTIP_ENABLE_SOUNDFX,
	}
}

local TOGGLES = {
	["error"] = {
		SoundOption	= SoundPanelOptions.Sound_EnableErrorSpeech,
		EnableCVar	= "Sound_EnableErrorSpeech",
		Enable		= AudioOptionsSoundPanelErrorSpeech,
		Tooltip		= OPTION_TOOLTIP_ENABLE_ERROR_SPEECH,
	},
	["emote"] = {
		SoundOption	= SoundPanelOptions.Sound_EnableEmoteSounds,
		EnableCVar	= "Sound_EnableEmoteSounds",
		Enable		= AudioOptionsSoundPanelEmoteSounds,
		Tooltip		= OPTION_TOOLTIP_ENABLE_EMOTE_SOUNDS,
	},
	["pet"] = {
		SoundOption	= SoundPanelOptions.Sound_EnablePetSounds,
		EnableCVar	= "Sound_EnablePetSounds",
		Enable		= AudioOptionsSoundPanelPetSounds,
		Tooltip		= OPTION_TOOLTIP_ENABLE_PET_SOUNDS,
	},
	["loop"] = {
		SoundOption	= SoundPanelOptions.Sound_ZoneMusicNoDelay,
		EnableCVar	= "Sound_ZoneMusicNoDelay",
		Enable		= AudioOptionsSoundPanelLoopMusic,
		Tooltip		= OPTION_TOOLTIP_ENABLE_MUSIC_LOOPING,
	},
	["background"] = {
		SoundOption	= SoundPanelOptions.Sound_EnableSoundWhenGameIsInBG,
		EnableCVar	= "Sound_EnableSoundWhenGameIsInBG",
		Enable		= AudioOptionsSoundPanelSoundInBG,
		Tooltip		= OPTION_TOOLTIP_ENABLE_BGSOUND,
	},
}

local HorizontalSliderBG = {
	bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
	edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
	edgeSize = 8, tile = true, tileSize = 8,
	insets = {left = 3, right = 3, top = 6, bottom = 6}
}
local function HideTooltip() GameTooltip:Hide() end
local function ShowTooltip(self)
	if not self.tooltip then return end
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetText(self.tooltip, nil, nil, nil, nil, true)
end

local function MakeCheckButton(parent)
	local check = CreateFrame("CheckButton", nil, parent)
	check:SetWidth(15)
	check:SetHeight(15)
	check:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
	check:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
	check:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
	check:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
	check:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")

	return check
end

local function MakeContainer(relative, dist)
	local container = CreateFrame("Frame", nil, Volumizer)
	container:SetWidth(155)
	container:SetHeight(40)
	container:SetPoint("TOP", relative, 0, (relative == Volumizer) and -22 or (relative and dist or -30))

	return container
end

local MakeToggle, MakeControl
do
	local hooksecurefunc = _G.hooksecurefunc
	local BlizzardOptionsPanel_GetCVarSafe = _G.BlizzardOptionsPanel_GetCVarSafe

	function MakeToggle(name, relative)
		local ref = TOGGLES[name]
		local container = MakeContainer(relative, -15)
		local check = MakeCheckButton(container)
		check:SetPoint("LEFT", container, "LEFT")
		check:SetChecked(ref.Enable:GetValue())
		check:SetHitRectInsets(-10, -150, 0, 0)
		check:SetScript("OnClick",
				function(checkButton)
					ref.Enable:SetValue(check:GetChecked() and 1 or 0)
				end)
		check.tooltip = ref.Tooltip
		check:SetScript("OnEnter", ShowTooltip)
		check:SetScript("OnLeave", HideTooltip)

		local text = check:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		text:SetPoint("LEFT", check, "RIGHT", 0, 3)
		text:SetText(_G[ref.SoundOption.text])

		hooksecurefunc("SetCVar",
			       function(cvar, value)
				       if cvar == ref.EnableCVar then
					       check:SetChecked(value)
				       end
			       end)
		return container
	end
	local function SetSliderLabel(slider, ref, value)
		slider.text:SetFormattedText("%s %d%%", _G[ref.SoundOption.text], value * 100)
	end
	function MakeControl(name, relative)
		local ref = VOLUMES[name]
		local container = MakeContainer(relative)
		local check = MakeCheckButton(container)
		check:SetPoint("LEFT", container, "LEFT")
		check:SetChecked(ref.Enable:GetValue())
		check:SetScript("OnClick",
				function(checkButton)
					ref.Enable:SetValue(check:GetChecked() and 1 or 0)
				end)
		check.tooltip = ref.Tooltip
		check:SetScript("OnEnter", ShowTooltip)
		check:SetScript("OnLeave", HideTooltip)

		local slider = CreateFrame("Slider", nil, container)
		slider:SetPoint("LEFT", check, "RIGHT", 0, 0)
		slider:SetPoint("RIGHT")
		slider:SetHeight(15)
		slider:SetHitRectInsets(0, 0, -10, -10)
		slider:SetOrientation("HORIZONTAL")
		slider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
		slider:SetBackdrop(HorizontalSliderBG)
		slider:SetMinMaxValues(ref.SoundOption.minValue, ref.SoundOption.maxValue)
		slider:SetValue(_G.BlizzardOptionsPanel_GetCVarSafe(ref.VolumeCVar))
		slider:SetValueStep(0.05)
		slider:EnableMouseWheel(true)

		slider.text = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		slider.text:SetPoint("BOTTOM", slider, "TOP", 0, 3)

		SetSliderLabel(slider, ref, ref.Volume:GetValue())

		slider:SetScript("OnValueChanged",
				 function(self, value)
					 value = tonumber(("%.2f"):format(value))
					 ref.Volume:SetValue(value)

					 SetSliderLabel(self, ref, value)
				 end)

		slider:SetScript("OnMouseWheel",
				 function(self, delta)
					 local currentValue = tonumber(("%.2f"):format(self:GetValue()))
					 local minValue, maxValue = self:GetMinMaxValues()
					 local step = self:GetValueStep()

					 if delta > 0 then
						 local new_value = tonumber(("%.2f"):format(math.min(maxValue, currentValue + step)))
						 self:SetValue(new_value)
					 elseif delta < 0 then
						 local new_value = tonumber(("%.2f"):format(math.max(minValue, currentValue - step)))
						 self:SetValue(new_value)
					 end
				 end)

		_G.hooksecurefunc("SetCVar",
				  function(cvar, value)
					  if cvar == ref.VolumeCVar then
						  slider:SetValue(value)
					  elseif cvar == ref.EnableCVar then
						  check:SetChecked(value)
					  end
				  end)
		return container
	end
end


function Volumizer:PLAYER_ENTERING_WORLD()
	self:SetFrameStrata("MEDIUM")
	self:SetWidth(180)
	self:SetHeight(240)
	self:SetToplevel(true)
	self:EnableMouse(true)
	self:SetClampedToScreen(true)
	self:Hide()
	
	self:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
		tile = true, tileSize = 6, edgeSize = 6, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	self:SetBackdropColor(0,0,0,1);
	
	local relative = self
	do
	local widget
	for k, v in pairs(VOLUMES) do
		widget = MakeControl(k, relative)
		relative = widget
	end
	relative = MakeContainer(relative, -10)	-- Blank space in panel.

	for k, v in pairs(TOGGLES) do
		widget = MakeToggle(k, relative)
		relative = widget
	end
	relative = MakeContainer(relative, -20)	-- Blank space in panel.
	end
	do
		local old_x, old_y, click_time
		WorldFrame:HookScript("OnMouseDown",
				      function(frame, ...)
					      old_x, old_y = _G.GetCursorPosition()
					      click_time = _G.GetTime()
				      end)

		WorldFrame:HookScript("OnMouseUp",
				      function(frame, ...)
					      local x, y = _G.GetCursorPosition()

					      if not old_x or not old_y or not x or not y or not click_time then
						      self:Hide()
						      return
					      end

					      if (math.abs(x - old_x) + math.abs(y - old_y)) <= 5 and _G.GetTime() - click_time < 1 then
						      self:Hide()
					      end
				      end)
		table.insert(UISpecialFrames, "VolumizerPanel")
		SLASH_Volumizer1 = "/vol"
		SlashCmdList["Volumizer"] = function()
			Volumizer:Toggle(nil)
		end
	end

	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self.PLAYER_ENTERING_WORLD = nil
end

function Volumizer:initialize(self, level)
	if not level then return end
	local info = DropDown.info
	wipe(info)

	if level == 1 then
		wipe(info)		-- Blank space in menu.
		info.disabled = true
		UIDropDownMenu_AddButton(info, level)
		info.disabled = nil

		info.text = DEFAULTS
		info.func = nil
		info.arg1 = 0
		info.colorCode = "|cffffff00"
		UIDropDownMenu_AddButton(info, level)
	elseif level == 2 then
			wipe(info)
			info.arg1 = UIDROPDOWNMENU_MENU_VALUE

			info.text = USE
			info.func = nil
			UIDropDownMenu_AddButton(info, level)

			info.text = SAVE
			info.func = nil
			UIDropDownMenu_AddButton(info, level)

			info.text = NAME
			info.func = nil
			UIDropDownMenu_AddButton(info, level)
	end
end

do
	local function GetAnchor(frame)
		if not frame then
			return "BOTTOMRIGHT", UIParent, -130, 25
		end
	end
	function Volumizer:Toggle(anchor)
		if self:IsShown() then
			self:Hide()
		else
			self:ClearAllPoints()
			self:SetPoint(GetAnchor(anchor))
			self:Show()
		end
	end
end	-- do

----------------------------------------
--      Quicklaunch registration      --
----------------------------------------

local dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Ampere", {
	type = "launcher",
	text = "Config",
	icon = "Interface\\Icons\\Spell_Nature_StormReach",
	OnClick = function(_,b) 
		if b == "LeftButton" then
			if IsShiftKeyDown() then ToggleCVar("nameplateShowFriends")
			elseif IsControlKeyDown() then ToggleCVar("UnitNameFriendlyPlayerName")
			elseif IsAltKeyDown() then Volumizer:Toggle(self)
			else InterfaceOptionsFrame_OpenToCategory(frame)
			end
		elseif b == "RightButton" then
			if IsShiftKeyDown() then ToggleCVar("nameplateShowEnemies")
			elseif IsControlKeyDown() then ToggleCVar("UnitNameEnemyPlayerName")
			else 
				if IsAddOnLoaded("Dominos") then
					Dominos:ToggleLockedFrames()
				elseif IsAddOnLoaded("m_ActionBars") then
					m_ActionBars_Toggle_Extra_Bars()
				end
			end
		elseif b == "MiddleButton" then
			if IsAddOnLoaded("Dominos") then
				Dominos:ToggleBindingMode()
			elseif IsAddOnLoaded("m_ActionBars") then
				m_ActionBars.MouseOverBind()
			end
		end
	end,
	OnTooltipShow = function(tooltip)
      tooltip:AddLine("|cffffffffQuick configuration|r",1,1,1)
      tooltip:AddDoubleLine("  Interface configuration","[|cffffffffLeftClick|r]")
	  tooltip:AddDoubleLine("  Volume settings","[|cffffffffAlt+LeftClick|r]")
	if IsAddOnLoaded("Dominos") then
      tooltip:AddDoubleLine("  Dominos: configure bars","[|cffffffffRightClick|r]")
	  tooltip:AddDoubleLine("  Dominos: set key bindings","[|cffffffffMiddleClick|r]")
	end
	if IsAddOnLoaded("m_ActionBars") and not IsAddOnLoaded("Dominos") then
      tooltip:AddDoubleLine("  m_ActionBars: toggle extra bars","[|cffffffffRightClick|r]")
	  tooltip:AddDoubleLine("  m_ActionBars: set key bindings","[|cffffffffMiddleClick|r]")
	end
	  tooltip:AddLine("|cffffffffPlayer Names|r")
      tooltip:AddDoubleLine("  Friendly player names","[|cffffffffCtrl+LeftClick|r]")
      tooltip:AddDoubleLine("  Enemy player names","[|cffffffffCtrl+RightClick|r]")
	  tooltip:AddLine("|cffffffffNameplates|r")
	  tooltip:AddDoubleLine("  Toggle friendly nameplates","[|cffffffffShift+LeftClick|r]")
      tooltip:AddDoubleLine("  Toggle enemy nameplates","[|cffffffffShift+RightClick|r]")
      tooltip:Show()
   end
})

function ToggleCVar(var)
   SetCVar(var, GetCVar(var) == "1" and 0 or 1)
   ChatFrame1:AddMessage("|cffcfb53b"..var.."|r is now "..(GetCVar(var) == "1" and "|cff00ff00shown|r" or "|cffff0000hidden|r")..".")
end

--[[-----------------------------------------------------------------------------
Broker_Durability
-------------------------------------------------------------------------------]]

local Dura = _G.CreateFrame("Frame", "Broker_Durability")
Dura.obj = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Durability", {
	value = "0", suffix = "%", text = "0%",
	icon = "Interface\\Icons\\Trade_BlackSmithing",
	}
)

_G.DurabilityFrame:UnregisterAllEvents()
_G.DurabilityFrame:Hide()

Dura:RegisterEvent("PLAYER_DEAD")
Dura:RegisterEvent("MERCHANT_CLOSED")
Dura:RegisterEvent("PLAYER_REGEN_ENABLED")
Dura:RegisterEvent("PLAYER_ENTERING_WORLD")

local itemslots = {
	"HeadSlot",
	"ShoulderSlot",
	"ChestSlot",
	"WristSlot",
	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"MainHandSlot",
	"SecondaryHandSlot",
}

local function updateItem(slotName)
	local slotId = GetInventorySlotInfo(slotName)

	local itemLink = GetInventoryItemLink("player", slotId)
	if not itemLink then
		return -1
	end

	local durability, maximum = GetInventoryItemDurability(slotId)
	 if not durability or (maximum == 0)then
		return -1
	end

	return (durability / maximum)
end

Dura:SetScript("OnEvent", function()
	local durabilityValue = 1
	local itemCounter = 0
	local durability = -1

	for _,slotName in ipairs(itemslots) do
		durability = updateItem(slotName)
		if durability >= 0 then
			durabilityValue = min(durabilityValue, durability)
			itemCounter = itemCounter + 1
		end
	end

	if itemCounter == 0 then return end

	local tDura = format("%i", floor(durabilityValue * 100))
	if floor(durabilityValue * 100) < 20 then
		Dura.obj.text = "|cffFF0000"..tDura.."%|r"
	else
		Dura.obj.text = tDura.."%"
	end
	Dura.obj.value = tDura
end)

local function FormatCurrency(amount)
	local gold = floor(amount / 10000)
	local silver = floor((amount - (gold * 10000)) / 100)
	local copper = mod(amount, 100)

	return format("%i|cffffd700g|r %i|cffc7c7cfs|r %i|cffeda55fc|r", gold, silver, copper)
end

local myTip = nil
function Dura.obj.OnTooltipShow(tip)
	if not tip or not tip.AddLine or not tip.AddDoubleLine then return end

	if not myTip then myTip = _G.CreateFrame("GameTooltip", "Broker_DurabilityTip") end
	local wornRepairCost, bagRepairCost, totalRepairCost = 0, 0, 0

	for _,slotName in ipairs(itemslots) do
		local item = _G["Character" .. slotName]
		local hasItem, _, repairCost
		if item then
			hasItem, _, repairCost = myTip:SetInventoryItem("player", item:GetID()) 
		end

		if hasItem and repairCost and repairCost > 0 then
			wornRepairCost = wornRepairCost + repairCost
		end
	end

	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local hasCooldown, repairCost = myTip:SetBagItem(bag, slot)

			if repairCost and repairCost > 0 then
				bagRepairCost = bagRepairCost + repairCost
			end
		end
	end

	totalRepairCost = wornRepairCost + bagRepairCost

	tip:AddLine('|cffFFFFFF'.._G["REPAIR_COST"]..'|r')
	tip:AddDoubleLine(_G["CURRENTLY_EQUIPPED"], FormatCurrency(wornRepairCost))
	tip:AddDoubleLine(_G["REPAIR_ALL_ITEMS"], FormatCurrency(totalRepairCost))
end


--[[-----------------------------------------------------------------------------
Broker_FPS
-------------------------------------------------------------------------------]]
local fps = {}
local lbl = " fps"
local format = _G.string.format
local GetFramerate = _G.GetFramerate
fps.obj = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Broker_FPS", {value = "0"..lbl, text = "0"..lbl, icon = "Interface\\AddOns\\m_BrokerStuff\\media\\Broker_FPS.tga"})
_G.LibStub("AceTimer-3.0"):Embed(fps)

local function green(x)  return '|cff00ff00'..x..'|r' end
local function red(x)    return '|cffF5591C'..x..'|r' end
local function yellow(x) return '|cffffff00'..x..'|r' end

function fps:Update()
	local rate = GetFramerate()
	local sfps = ('%.1f'):format(rate)
	if rate <= 20 then
		self.obj.text = sfps..lbl
		self.obj.value = red(sfps)..lbl
	elseif rate <= 29 then
		self.obj.text = sfps..lbl
		self.obj.value = yellow(sfps)..lbl
	else
		self.obj.text = sfps..lbl
		self.obj.value = green(sfps)..lbl
	end

end

fps:ScheduleRepeatingTimer("Update", 2)

--[[-----------------------------------------------------------------------------
Broker_Latency
-------------------------------------------------------------------------------]]
local latency = {}
local lbl = " ms"
local format, modf, GetNetStats = _G.string.format, math.modf, GetNetStats
latency.obj = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Broker_Latency", {value = "0"..lbl .." | ".."0"..lbl, text = "0"..lbl .." | ".."0"..lbl}) -- one obj for home and world
_G.LibStub("AceTimer-3.0"):Embed(latency) 

local function green(x)  return '|cff00ff00'..x..'|r' end
local function red(x)    return '|cffF5591C'..x..'|r' end
local function yellow(x) return '|cffffff00'..x..'|r' end 

function latency:Update()
	local _, _, l_home, l_world = _G.GetNetStats() -- get home and world
	local sl = ('%.1f'):format(l_home)
	local s2 = ('%.1f'):format(l_world)
	-- temp stuff
	local text_home_temp = ""
	local value_home_temp = ""
	local text_world_temp = ""
	local value_world_temp = ""
	-- home
	if l_home >= 300 then
		text_home_temp = sl..lbl
		value_home_temp = red(sl)..lbl
	elseif l_home >= 100 then
		text_home_temp = sl..lbl
		value_home_temp = yellow(sl)..lbl
	else
		text_home_temp = sl..lbl
		value_home_temp = green(sl)..lbl
	end
	-- world
	if l_world >= 300 then
		text_world_temp = s2..lbl
		value_world_temp = red(s2)..lbl
	elseif l_world >= 100 then
		text_world_temp = s2..lbl
		value_world_temp = yellow(s2)..lbl
	else
	text_world_temp = s2..lbl
	value_world_temp = green(s2)..lbl
	end
	-- merge the vars
	self.obj.text = text_home_temp .." | ".. text_world_temp
	self.obj.value = value_home_temp .." | ".. value_world_temp
end

local function ColorGradient(perc, r1, g1, b1, r2, g2, b2, r3, g3, b3)
	if perc >= 1 then return r3, g3, b3 elseif perc <= 0 then return r1, g1, b1 end
	local segment, relperc = modf(perc*2)
	if segment == 1 then r1, g1, b1, r2, g2, b2 = r2, g2, b2, r3, g3, b3 end
	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

function latency.obj.OnTooltipShow(self)
	GameTooltip:AddLine("Latency and bandwidth usage",1,1,1)

	local binz, boutz, _, _ = GetNetStats()
	local rin, gin, bins = ColorGradient(binz/20, 0,1,0, 1,1,0, 1,0,0)
	local rout, gout, bout = ColorGradient(boutz/5, 0,1,0, 1,1,0, 1,0,0)
	
	GameTooltip:AddDoubleLine("|cffFF7A38Incoming|r bandwidth usage:", format("%.2f kb/sec", binz), 1, 1, 1, rin, gin, bins)
	GameTooltip:AddDoubleLine("|cff06ddfaOutgoing|r bandwidth usage:", format("%.2f kb/sec", boutz), 1, 1, 1, rout, gout, bout)
	GameTooltip:Show()

	GameTooltip:AddDoubleLine("\nLatency", "\n"..latency.obj.value, 1, 1, 1, 0, 1, 0) -- i dont know whether it wants a string or an int just try
end

latency:ScheduleRepeatingTimer("Update", 30)
latency:ScheduleTimer("Update", 5)

--[[-----------------------------------------------------------------------------
Broker_Memory
-------------------------------------------------------------------------------]]
local L_MB = "mb"
local L_KB = "kb"
local Mem = {}
local MemUse
local format = string.format
Mem.obj = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Broker_Memory", {value = "0"..L_MB, text = "0 "..L_MB})
_G.LibStub("AceTimer-3.0"):Embed(Mem)

local function green(x)  return '|cff00ff00'..x..'|r' end

function Mem:Update()
	local current = format("%.1f", _G.collectgarbage("count") / 1024)
	MemUseCalc()
	Mem.obj.text = MemUse
	Mem.obj.value = green(MemUse)
end

Mem:ScheduleRepeatingTimer("Update", 5)
Mem:ScheduleTimer("Update", 5)

function Mem.obj.OnClick()
	_G.GameTooltip:Hide()
	_G.collectgarbage("collect")
	Mem:Update()
end

local function formatMemory(n)
	if n > 1024 then
		return format("%.2f "..L_MB, n / 1024)
	else
		return format("%.2f "..L_KB, n)
	end
end

local function mySort(x,y)
	return x.mem > y.mem
end

local memTbl = {}
function Mem.obj.OnTooltipShow(tooltip)
	if not tooltip or not tooltip.AddLine or not tooltip.AddDoubleLine then return end
	tooltip:AddLine("Memory usage", 1, 1, 1)
	UpdateAddOnMemoryUsage()
	local grandtotal = collectgarbage("count")
	local total = 0

	local tinsert = _G.table.insert
	local IsAddOnLoaded = _G.IsAddOnLoaded
	local GetAddOnMemoryUsage = _G.GetAddOnMemoryUsage
	local GetAddOnInfo = _G.GetAddOnInfo
	for i = 1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local memused = GetAddOnMemoryUsage(i)
			total = total + memused
			tinsert(memTbl, {addon = GetAddOnInfo(i), mem = memused})
		end
	end
	ii = 0
	table.sort(memTbl, mySort)
	local txt = "%d. %s"
	local killPoint = tonumber(_G.SB_MEM_KILL) or 0
	for k, v in _G.pairs(memTbl) do
		tooltip:AddDoubleLine(format(txt, k, v.addon), formatMemory(v.mem), 1, 1, 1, 0, 1, 0)
		if k == killPoint then break end
		if ii >= 69 then	break end
		ii = ii + 1
	end
	for i = 1, #memTbl do memTbl[i] = nil end

	tooltip:AddDoubleLine("\nTotal", "\n"..formatMemory(total), 1, 1, 1, 0, 1, 0)
	tooltip:AddDoubleLine("Total & Blizzard AddOns", formatMemory(grandtotal), 1, 1, 1, 0, 1, 0)
	tooltip:AddLine("|cffeda55fClick|r to collect garbage", 1, 1, 1)
	MemUse = total
end

function MemUseCalc()
	UpdateAddOnMemoryUsage()
	local total = 0

	local IsAddOnLoaded = _G.IsAddOnLoaded
	local GetAddOnMemoryUsage = _G.GetAddOnMemoryUsage
	local GetAddOnInfo = _G.GetAddOnInfo
	for i = 1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local memused = GetAddOnMemoryUsage(i)
			total = total + memused
		end
	end
	
	if total > 1024 then
		MemUse = format("%.2f "..L_MB, total / 1024)
	else
		MemUse = format("%.1f "..L_KB, total)
	end
	
end

--[[-----------------------------------------------------------------------------
Broker_Money
-------------------------------------------------------------------------------]]
local Object = LibStub("LibDataBroker-1.1"):NewDataObject("Money", { ["type"] = "data source" })
local Frame, Name, Realm = CreateFrame("Frame"), UnitName("player"), GetRealmName()

Frame:RegisterEvent("PLAYER_LOGIN")
Frame:RegisterEvent("PLAYER_MONEY")
Frame:RegisterEvent("PLAYER_TRADE_MONEY")
Frame:RegisterEvent("TRADE_MONEY_CHANGED")
Frame:SetScript("OnEvent", function(_, Event)
	if Event == "PLAYER_LOGIN" then
		MoneyDB = MoneyDB or {}
		MoneyDB[Realm] = MoneyDB[Realm] or {}
	end
	
	MoneyDB[Realm][Name] = GetMoney()
	Object.text = floor(GetMoney()*.0001).. "|cffffd700 g|r"
end)

local session
local function OnEvent(self, event, addon)
	if event ~= "ADDON_LOADED" then
		if not session then session = GetMoney() end
	end
end
local f=CreateFrame"Frame"
f:SetScript( "OnEvent", OnEvent )
f:RegisterEvent"ADDON_LOADED"
f:RegisterEvent"PLAYER_MONEY"
f:RegisterEvent"PLAYER_TRADE_MONEY"
f:RegisterEvent"TRADE_MONEY_CHANGED"
f:RegisterEvent"SEND_MAIL_MONEY_CHANGED"
f:RegisterEvent"SEND_MAIL_COD_CHANGED"
f:RegisterEvent"PLAYER_LOGIN"
local function FormatMoney(value)
	return	(value > 9999 or value < -9999) and format("%i|cffffd700g|r %.2i|cffc7c7cfs|r %.2i|cffeda55fc|r", floor(value*.0001), floor(value%10000*.01), value%100 )
		or (value > 99 or value < -99) and format("%i|cffc7c7cfs|r %.2i|cffeda55fc|r", floor(value*.01), value%100 )
		or format("%i|cffeda55fc|r", value), value <=0 and 1 or 0, value < 0 and 0 or 1, 0, 1,1,1
end

Object.OnTooltipShow = function(Tooltip)
	local TotalServerMoney = 0
	
	Tooltip:AddLine("Money:", 1, 1, 1)
	local total = GetMoney()
	Tooltip:AddDoubleLine( "Session", FormatMoney( total - session ) )
	Tooltip:AddLine(" ")
	Tooltip:AddDoubleLine("Player:", "Money:", 1, 1, 1, 1, 1, 1)
	Tooltip:AddDoubleLine("--------------------", "--------------------", 1, 1, 1, 1, 1, 1)
	
	for PlayerName, PlayerMoney in next, MoneyDB[Realm] do
		Tooltip:AddDoubleLine(PlayerName, FormatMoney(PlayerMoney), 1, 1, 1, 1, 1, 1)
		TotalServerMoney = TotalServerMoney + PlayerMoney
	end
	
	Tooltip:AddDoubleLine("--------------------", "--------------------", 1, 1, 1, 1, 1, 1)
	Tooltip:AddLine(" ")
	Tooltip:AddDoubleLine("Total Money (" .. Realm .. "):", FormatMoney(TotalServerMoney), 1, 1, 1, 1, 1, 1)
	Tooltip:AddLine(" ")
	Tooltip:AddLine("|cffeda55fALT + Click|r to wipe gathered gold data", 1, 1, 1)
	Tooltip:AddLine("|cffeda55fClick|r to open your bags", 1, 1, 1)
end

function Object.OnClick()
	if IsAltKeyDown() then
		_G.GameTooltip:Hide()
		if(MoneyDB) then table.wipe(MoneyDB) 
			MoneyDB = MoneyDB or {}
			MoneyDB[Realm] = MoneyDB[Realm] or {}
			MoneyDB[Realm][Name] = GetMoney()
			Object.text = floor(GetMoney()*.0001).. " g"
		end
	else
		ToggleBackpack()
	end
end
